#ifndef GUI_ENV
#include <cutils.h>
#else
#include <utils.h>
#endif
#include <int_main.h>
#include <gtkmrcomune.h>
#include <gtkdefs.h>
#include "sir.h"
#include "forcomm1.h"
#include <forcomm2.h>
#include <forlink.h>
#include "forlink1.h"
#include "forlink2.h"

static const char *desc_about = "A program for automatic solution and refinement \nof crystal structures.\n\n"
"By M.C. Burla, R. Caliandro, B. Carrozzini, \nG.L. Cascarano, "
"C. Cuocci, C. Giacovazzo,\nM.Mallamo, A. Mazzone, G.Polidori,\nD.Siliqi.";
//static const char  *about_pixmap  = "sir2020about.png";
static const char  *about_pixmap  = MR_PREF"about.png";

// void SettaGruppoDefault(const char *nome);

namespace main_app {
Application TheApp = Application(MR_PACKAGE, MR_VERSION, PK, prg_files_dir,
                 EX_DIR, 1, desc_about, about_pixmap);

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
bool setCCP4env()
{
	char value[255];
	DWORD BufferSize = 8192;

	LSTATUS st =
		RegGetValueA(HKEY_LOCAL_MACHINE, "SOFTWARE\\WOW6432Node\\CCP4\\CCP4", "InstallDir", RRF_RT_REG_SZ, NULL, &value, &BufferSize);

	if (st == 0)
		return TRUE;
	st =
		RegGetValueA(HKEY_LOCAL_MACHINE, "SOFTWARE\\WOW6432Node\\CCP4-7", "InstallDir", RRF_RT_REG_SZ, NULL, &value, &BufferSize);
	if (st == 0)
	{
		std::cout << value << "\n";
		// TODO set CCP4 envirnoment
		std::string ccp4master(value);
		std::string str = "CCP4_MASTER=" + ccp4master;
		_putenv(str.c_str());
		_putenv("CCP4_SCR=c:\\ccp4temp");
		std::string ccp4;
		GDir *dir = g_dir_open(ccp4master.c_str(), 0, NULL);
		if (!dir)
			return false;
		const gchar *name = g_dir_read_name(dir);
		while (name != NULL)
		{
			gchar *path;
			gboolean is_dir;

			path = g_build_filename(ccp4master.c_str(), name, NULL);
			is_dir = g_file_test(path, G_FILE_TEST_IS_DIR);
			if(is_dir && (name[0] == '7'))
				ccp4.assign(path);
			g_free(path);
			name = g_dir_read_name(dir);
		}
		g_dir_close(dir);
		if (ccp4.empty())
			return false;
		str.assign("CCP4=" + ccp4);
		_putenv(str.c_str());
		str.assign("CBIN=");
		str.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "bin", NULL));
		_putenv(str.c_str());
		str.assign("CLIB=");
		str.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "lib", NULL));
		_putenv(str.c_str());
		str.assign("CLIBD=");
		str.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "lib", "data", NULL));
		_putenv(str.c_str());
		str.assign("CEXAM=");
		str.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "examples", NULL));
		_putenv(str.c_str());
		str.assign("CHTML=");
		str.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "html", NULL));
		_putenv(str.c_str());
		str.assign("CINCL=");
		str.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "include", NULL));
		_putenv(str.c_str());
		str.assign("CLIBD_MON=");
		str.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "lib", "data", "monomers", NULL));
		str.append("\\");
		_putenv(str.c_str());
		str.assign("CHTML=");
		str.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "html", NULL));
		_putenv(str.c_str());
		std::string str_path = Ambiente("PATH");
		if (!str_path.empty())
		{
			std::string path = "PATH=";
			path.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "nodejs", NULL));
			path.append(";");
			path.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "bin", NULL));
			path.append(";");
			path.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "etc", NULL));
			path.append(";");
			path.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "Scripts", NULL));
			path.append(";");
			path.append(g_build_path(G_DIR_SEPARATOR_S, ccp4.c_str(), "etc", NULL));
			path.append(";");
			path.append(str_path);
			_putenv(path.c_str());
			std::cout << path << "\n";
		}
		return true;
	}
	std::cout << st << "\n";

	return false;
}
#endif

int Main (int argc, char **argv)
{
#ifdef _WIN32
	bool result1 = setCCP4env();
#endif

     int k = 0;
     int i, result = TheApp.Initialize(argc, argv); 

     if(!result)
        return -1;

     TheApp.standalone = true;
     dialogo->NOCFG = 0;         //22/07/2016
     args->argc = argc;
     for(i=0; i<argc; i++)
     {
         if(strcmp(argv[i], "--version") == 0)
         {
            TheApp.InfoAbout();
            return 0;
         }
         else if(strcmp(argv[i], "--sub") == 0)
         {
            args->argc--;
            TheApp.standalone = false;
         }
//22/07/2016
         else if(strcmp(argv[i], "--nocfg") == 0)
         {
            args->argc--;
            TheApp.nocfg = true;
            dialogo->NOCFG = 1;
         }
//END 22/07/2016
         else
         {
            CopiaStringa(args->arg[k], MaxLenFilename, argv[i]);
            k++;
         }
     }

     dialogo->ifGTK        = 1;

     ScriviStringa(dialogo->ProgName, MaxLenFilename, "%s", TheApp.PackageName());
     dialogo->lProgName = (int) strlen(dialogo->ProgName);
     ScriviStringa(dialogo->PkgName, MaxLenFilename, "%s", PACKAGE_STRING);
     dialogo->lPkgName = (int) strlen(dialogo->PkgName);
//     SettaGruppoDefault(TheApp.GroupsFile().c_str());
     sir2006main_();

    return 0;
}

#ifdef __cplusplus
          }
#endif
}

std::string main_dir()
{
    std::string dir = main_app::TheApp.MainDir();
    return dir;
}

extern "C" void leggiconfig_()
{
     main_app::TheApp.LeggiConfig();
     ScriviStringa(SWareRC->SWRC, MaxLenFilename, "%s", main_app::TheApp.ConfFile().c_str());
     ScriviStringa(SWareRC->HOMEdir, MaxLenFilename, "%s", main_app::TheApp.ConfDir().c_str());
     ScriviStringa(SWareRC->WorkingDirectory, MaxLenFilename, "%s", main_app::TheApp.WorkingDirectory().c_str());
     RP->nRecentProjects   = main_app::TheApp.nRecentProjects;
     for(int i=0; i<5;i++)
         CopiaStringa(RP->RecentProj[i], MaxLenFilename, main_app::TheApp.RecentProj[i]);
     dialogo->ifLabels     = main_app::TheApp.IfLabels;
     dialogo->iPlotType    = main_app::TheApp.PlotType;
     dialogo->ifBestVisual = main_app::TheApp.Bestv;
     dialogo->color_GHOST        = GHOST;
     dialogo->color_CELL         = CELLC;
     dialogo->jBackGround        = main_app::TheApp.Background;
}
